"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);

// src/lib/index.ts
var lib_exports = {};
__export(lib_exports, {
  createUseQueryParams: () => createUseQueryParams
});
module.exports = __toCommonJS(lib_exports);

// src/lib/create-params.svelte.ts
var import_svelte = require("svelte");

// src/lib/adapters/dom.ts
function createQueryFetcher(windowObj) {
  return () => windowObj.location;
}
function createQueryUpdater(replace, { replaceState, pushState }) {
  const update = replace ? replaceState : pushState;
  return (search, hash) => update(null, "", `${search}${hash}`);
}
function dom(options = {}) {
  const { windowObj = window, replace = false } = options;
  const replaceState = windowObj.history.replaceState.bind(windowObj.history);
  const pushState = windowObj.history.pushState.bind(windowObj.history);
  return {
    isBrowser: () => typeof window !== "undefined",
    getBrowserUrl: createQueryFetcher(windowObj),
    updateBrowserUrl: createQueryUpdater(replace, { replaceState, pushState }),
    // Client-side only adapter
    getServerUrl: () => ({ hash: "", search: "" }),
    updateServerUrl: () => {
    }
  };
}

// src/lib/utils.ts
var import_valibot = require("valibot");
function parseValue(schema, value) {
  if (typeof schema === "function") {
    return schema(value);
  } else if ("parse" in schema) {
    return schema.parse(value);
  } else if ("async" in schema) {
    return (0, import_valibot.parse)(schema, value);
  }
  return value;
}
function parseQueryParams(params, schemas) {
  const clone = {};
  const keys = /* @__PURE__ */ new Set([...Object.keys(params), ...Object.keys(schemas)]);
  const log = console.log;
  console.log = () => {
  };
  console.log({ params });
  console.log = log;
  for (const key of keys) {
    const value = params[key];
    clone[key] = key in schemas ? parseValue(schemas[key], value) : (
      /** Value wasn't defined in the schema, pass through as-is */
      value
    );
  }
  return clone;
}
function createDefaultSerializer() {
  return (value) => typeof value === "string" ? value : JSON.stringify(value);
}
function entriesToRecord(entries) {
  return entries.reduce((acc, [key, value]) => {
    acc[key] = value;
    return acc;
  }, {});
}
function mapValues(object, mapFn) {
  return Object.fromEntries(
    Object.entries(object).map(([key, value]) => [key, mapFn(value)])
  );
}
function debounce(func, delay) {
  if (delay === 0)
    return func;
  let timeoutId;
  return function(...args) {
    clearTimeout(timeoutId);
    timeoutId = setTimeout(() => {
      func(...args);
    }, delay);
  };
}

// src/lib/create-params.svelte.ts
function createUseQueryParams(validators, options = {}) {
  const {
    debounce: delay = 0,
    windowObj = typeof window === "undefined" ? void 0 : window,
    adapter = dom({ windowObj }),
    serialise = createDefaultSerializer()
  } = options;
  let raw = $state(getQueryParams());
  const query = $derived(parseQueryParams(raw, validators));
  const search = $derived(
    Object.keys(query).length ? `?${new URLSearchParams(query)}` : ""
  );
  function getQueryParams() {
    const getParams = adapter.isBrowser() ? adapter.getBrowserUrl : adapter.getServerUrl;
    const { search: search2 } = getParams();
    const queryParams = new URLSearchParams(search2);
    return entriesToRecord(Array.from(queryParams));
  }
  const updateUrl = debounce(async () => {
    await (0, import_svelte.tick)();
    if (adapter.isBrowser()) {
      adapter.updateBrowserUrl(search, adapter.getBrowserUrl().hash);
    } else {
      adapter.updateServerUrl(search, adapter.getServerUrl().hash);
    }
  }, delay);
  function updateQueryParams() {
    raw = getQueryParams();
  }
  function setQueryParam(key, value) {
    if (value === void 0) {
      const { [key]: _, ...rest } = raw;
      raw = rest;
    } else {
      raw = { ...raw, [key]: serialise(value) };
    }
    updateUrl();
  }
  let replaceState;
  let pushState;
  if (windowObj) {
    replaceState = windowObj.history.replaceState.bind(windowObj.history);
    pushState = windowObj.history.pushState.bind(windowObj.history);
    windowObj.history.replaceState = function(...args) {
      replaceState(...args);
      updateQueryParams();
    };
    windowObj.history.pushState = function(...args) {
      pushState(...args);
      updateQueryParams();
    };
    windowObj.addEventListener("popstate", updateQueryParams);
  }
  const reactive = {};
  for (const key of Object.keys(validators)) {
    Object.defineProperty(reactive, key, {
      get() {
        return query[key];
      },
      set(newValue) {
        setQueryParam(key, newValue);
      }
    });
  }
  Object.defineProperties(reactive, {
    raw: {
      get() {
        return raw;
      }
    },
    query: {
      get() {
        return query;
      }
    },
    search: {
      get() {
        return search;
      }
    }
  });
  return () => Object.assign(reactive, {
    keys() {
      return Object.keys(validators);
    },
    entries() {
      return Object.entries(validators).map(([key]) => [key, query[key]]);
    },
    set(params) {
      raw = mapValues(params, serialise);
      updateUrl();
    },
    update(params) {
      raw = { ...raw, ...mapValues(params, serialise) };
      updateUrl();
    },
    remove(...params) {
      raw = Object.fromEntries(
        Object.entries(raw).filter(([key]) => !params.includes(key))
      );
    },
    unsubscribe() {
      if (windowObj) {
        windowObj.removeEventListener("popstate", updateQueryParams);
        windowObj.history.pushState = pushState;
        windowObj.history.replaceState = replaceState;
      }
    },
    [Symbol.dispose]() {
      this.unsubscribe();
    }
  });
}
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  createUseQueryParams
});
//# sourceMappingURL=index.svelte.cjs.map